//********************************************************************
//*** ResourcePool/oracle/CommandInterface.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: CommandInterface.hh,v 1.12 2005/05/07 19:50:45 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_HH
#define RESOURCEPOOL_ORACLE_COMMANDINTERFACE_HH

#ifndef RESOURCEPOOL_ORACLE_RESOURCEFWD_HH
#include "ResourceFwd.hh"
#endif

#ifndef RESOURCEPOOL_COMMAND_HH
#include "ResourcePool/Command.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_STATEMENTCACHE_HH
#include "StatementCache.hh"
#endif

namespace fatalmind {
namespace oracle {

/*
 * this class implements the non template specialized parts 
 * which are common to ALL oracle commands (SQL based as well
 * as Transaction)
 *
 * currently this manages the auto commit stuff only. this is on
 * per default and managed for transactions. not available to the user.
 * 
 * public interfaces since it will be inherited protected
 */
class CommandBase 
{
	public:
		CommandBase() 
		: _ac(true)
		{
		}

		virtual ~CommandBase() {
		}

		/*
		 * needs to be virtual because Transaction is overloading.
		 * to make this work for recursive transactions as well
		 */
		virtual void autoCommit(bool ac = true) {
			_ac = ac;
		}	

		/*
		 * this is implemented in CommandInterface.cc
		 * which is included in the .cc files which actually use it
		 * so that it can be inlined
		 */
		inline int _ac_mode() const; 

	private:
		bool _ac;
};

template<class TM, template<class> class SLMDC>
class Transaction;

/**
 * this is actually the class which the CommandCommon and Transaction
 * inherit from (public)
 *
 * Transaction needs to be friend to be allowed to access the protected
 * methods
 */
template<class TM, template<class> class SLMDC>
class CommandInterface
: public Command<fatalmind::oracle::ResourceType<TM, SLMDC> > 
, protected CommandBase
{
	public:
		virtual ~CommandInterface() {
		};
	protected:
		virtual CommandInterface<TM, SLMDC>* clone() const = 0;
		virtual void _execute(oracleHandlesWithCache<TM>&, StatementCache&) = 0;
	friend
		class Transaction<TM, SLMDC>;
};

} // namespace oracle
} // namespace fatalmind
#endif
