//********************************************************************
//*** ResourcePool/oracle/Transaction.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Transaction.hh,v 1.11 2005/01/16 10:39:04 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_TRANSACTION_HH
#define RESOURCEPOOL_ORACLE_TRANSACTION_HH

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_HH
#include "CommandInterface.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_STATEMENTCACHE_HH
#include "StatementCache.hh"
#endif

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#include <vector>

namespace fatalmind {
namespace oracle {

/**
 * This class implements the concept of a database transaction.
 * It is a container for other Commands (SQLExecute, or whatever).
 * All those commands will be executed within one transaction which
 * is automatically committed on success. On failure (every exception)
 * a rollback is performed.
 */
template<class TM, template<class> class SLMDC>
class Transaction: public CommandInterface<TM, SLMDC> {
	public:
		/**
		 * Used to create a Transaction command.
		 */
		Transaction();
		virtual ~Transaction();

		/**
		 * Adds a command to the transaction. execution is done in order
		 * of adding. Even Transaction commands can be put into an
		 * Transaction, but this does NOT cause nested transactions to be
		 * used on the database (aka. savepoints) but there will be one
		 * transaction including everything.
		 * TODO: make nested transactions possible ;)
		 */
		void add(const CommandInterface<TM, SLMDC>&);
		virtual void execute(oracleHandlesWithCache<TM>&);
		void autoCommit(bool ac = true);	
		virtual CommandInterface<TM, SLMDC>* clone() const;
	protected:
		virtual void _execute(oracleHandlesWithCache<TM>&, StatementCache&);
		void commit(oracleHandlesWithCache<TM>&);
		void rollback(oracleHandlesWithCache<TM>&);
	private:
		void doit(oracleHandlesWithCache<TM>& hand, StatementCache&);
		typedef std::vector<gc_ptr<CommandInterface<TM, SLMDC> > > _list_t;
		_list_t _list;
};

} // namespace oracle
} // namespace fatalmind
#endif
